==> drill-java-exec-1.12.0.jar: org/apache/drill/exec/expr/fn/impl/GCompareFloat4VsFloat4.java <==

/*******************************************************************************

 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.drill.exec.expr.fn.impl;

import org.apache.drill.exec.expr.DrillSimpleFunc;
import org.apache.drill.exec.expr.annotations.FunctionTemplate;
import org.apache.drill.exec.expr.annotations.FunctionTemplate.NullHandling;
import org.apache.drill.exec.expr.annotations.Output;
import org.apache.drill.exec.expr.annotations.Param;
import org.apache.drill.exec.expr.fn.FunctionGenerationHelper;
import org.apache.drill.exec.expr.fn.impl.ByteFunctionHelpers;
import org.apache.drill.exec.expr.holders.*;
import org.apache.drill.exec.record.RecordBatch;
import javax.inject.Inject;
import io.netty.buffer.DrillBuf;

/*
 * This class is generated using freemarker and the ComparisonFunctions.java template.
 */

@SuppressWarnings("unused")
public class GCompareFloat4VsFloat4 {



  @FunctionTemplate(name = FunctionGenerationHelper.COMPARE_TO_NULLS_HIGH,
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.INTERNAL)
  public static class GCompareFloat4VsFloat4NullHigh implements DrillSimpleFunc {

    @Param Float4Holder left;
    @Param Float4Holder right;
    @Output IntHolder out;

    public void setup() {}

    public void eval() {
     outside:
      {



      out.value = left.value < right.value ? -1 : (left.value == right.value ? 0 : 1);

      } // outside
    }
  }

  @FunctionTemplate(name = FunctionGenerationHelper.COMPARE_TO_NULLS_LOW,
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.INTERNAL)
  public static class GCompareFloat4VsFloat4NullLow implements DrillSimpleFunc {

    @Param Float4Holder left;
    @Param Float4Holder right;
    @Output IntHolder out;

    public void setup() {}

    public void eval() {
     outside:
      {



      out.value = left.value < right.value ? -1 : (left.value == right.value ? 0 : 1);

      } // outside
    }
  }



  @FunctionTemplate(name = FunctionGenerationHelper.COMPARE_TO_NULLS_HIGH,
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.INTERNAL)
  public static class GCompareFloat4VsNullableFloat4NullHigh implements DrillSimpleFunc {

    @Param Float4Holder left;
    @Param NullableFloat4Holder right;
    @Output IntHolder out;

    public void setup() {}

    public void eval() {
     outside:
      {

      if ( right.isSet == 0 ) {
        out.value = -1;
        break outside;
      }


      out.value = left.value < right.value ? -1 : (left.value == right.value ? 0 : 1);

      } // outside
    }
  }

  @FunctionTemplate(name = FunctionGenerationHelper.COMPARE_TO_NULLS_LOW,
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.INTERNAL)
  public static class GCompareFloat4VsNullableFloat4NullLow implements DrillSimpleFunc {

    @Param Float4Holder left;
    @Param NullableFloat4Holder right;
    @Output IntHolder out;

    public void setup() {}

    public void eval() {
     outside:
      {

      if ( right.isSet == 0 ) {
        out.value = 1;
        break outside;
      }


      out.value = left.value < right.value ? -1 : (left.value == right.value ? 0 : 1);

      } // outside
    }
  }



  @FunctionTemplate(name = FunctionGenerationHelper.COMPARE_TO_NULLS_HIGH,
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.INTERNAL)
  public static class GCompareNullableFloat4VsFloat4NullHigh implements DrillSimpleFunc {

    @Param NullableFloat4Holder left;
    @Param Float4Holder right;
    @Output IntHolder out;

    public void setup() {}

    public void eval() {
     outside:
      {

        if ( left.isSet == 0 ) {
          out.value = 1;
          break outside;
        }


      out.value = left.value < right.value ? -1 : (left.value == right.value ? 0 : 1);

      } // outside
    }
  }

  @FunctionTemplate(name = FunctionGenerationHelper.COMPARE_TO_NULLS_LOW,
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.INTERNAL)
  public static class GCompareNullableFloat4VsFloat4NullLow implements DrillSimpleFunc {

    @Param NullableFloat4Holder left;
    @Param Float4Holder right;
    @Output IntHolder out;

    public void setup() {}

    public void eval() {
     outside:
      {

        if ( left.isSet == 0 ) {
          out.value = -1;
          break outside;
        }


      out.value = left.value < right.value ? -1 : (left.value == right.value ? 0 : 1);

      } // outside
    }
  }



  @FunctionTemplate(name = FunctionGenerationHelper.COMPARE_TO_NULLS_HIGH,
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.INTERNAL)
  public static class GCompareNullableFloat4VsNullableFloat4NullHigh implements DrillSimpleFunc {

    @Param NullableFloat4Holder left;
    @Param NullableFloat4Holder right;
    @Output IntHolder out;

    public void setup() {}

    public void eval() {
     outside:
      {

        if ( left.isSet == 0 ) {
          if ( right.isSet == 0 ) {
            out.value = 0;
            break outside;
          } else {
            out.value = 1;
            break outside;
          }
        } else if ( right.isSet == 0 ) {
          out.value = -1;
          break outside;
        }


      out.value = left.value < right.value ? -1 : (left.value == right.value ? 0 : 1);

      } // outside
    }
  }

  @FunctionTemplate(name = FunctionGenerationHelper.COMPARE_TO_NULLS_LOW,
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.INTERNAL)
  public static class GCompareNullableFloat4VsNullableFloat4NullLow implements DrillSimpleFunc {

    @Param NullableFloat4Holder left;
    @Param NullableFloat4Holder right;
    @Output IntHolder out;

    public void setup() {}

    public void eval() {
     outside:
      {

        if ( left.isSet == 0 ) {
          if ( right.isSet == 0 ) {
            out.value = 0;
            break outside;
          } else {
            out.value = -1;
            break outside;
          }
        } else if ( right.isSet == 0 ) {
          out.value = 1;
          break outside;
        }


      out.value = left.value < right.value ? -1 : (left.value == right.value ? 0 : 1);

      } // outside
    }
  }

   @FunctionTemplate(names = {"less_than", "<"},
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.NULL_IF_NULL)
  public static class LessThanFloat4VsFloat4 implements DrillSimpleFunc {

      @Param Float4Holder left;
      @Param Float4Holder right;
      @Output BitHolder out;

      public void setup() {}

      public void eval() {

          out.value = left.value < right.value ? 1 : 0;

      }
  }

  @FunctionTemplate(names = {"less_than_or_equal_to", "<="},
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.NULL_IF_NULL)
  public static class LessThanEqFloat4VsFloat4 implements DrillSimpleFunc {

      @Param Float4Holder left;
      @Param Float4Holder right;
      @Output BitHolder out;

      public void setup() {}

      public void eval() {

          out.value = left.value <= right.value ? 1 : 0;

    }
  }

  @FunctionTemplate(names = {"greater_than", ">"},
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.NULL_IF_NULL)
  public static class GreaterThanFloat4VsFloat4 implements DrillSimpleFunc {

      @Param Float4Holder left;
      @Param Float4Holder right;
      @Output BitHolder out;

      public void setup() {}

      public void eval() {

          out.value = left.value > right.value ? 1 : 0;

    }
  }

  @FunctionTemplate(names = {"greater_than_or_equal_to", ">="},
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.NULL_IF_NULL)
  public static class GreaterThanEqFloat4VsFloat4 implements DrillSimpleFunc {

      @Param Float4Holder left;
      @Param Float4Holder right;
      @Output BitHolder out;

      public void setup() {}

      public void eval() {

          out.value = left.value >= right.value ? 1 : 0;

      }
  }

  @FunctionTemplate(names = {"equal", "==", "="},
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.NULL_IF_NULL)
  public static class EqualsFloat4VsFloat4 implements DrillSimpleFunc {

      @Param Float4Holder left;
      @Param Float4Holder right;
      @Output BitHolder out;

      public void setup() {}

      public void eval() {

          out.value = left.value == right.value ? 1 : 0;

      }
  }

  @FunctionTemplate(names = {"not_equal", "<>", "!="},
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.NULL_IF_NULL)
  public static class NotEqualsFloat4VsFloat4 implements DrillSimpleFunc {

      @Param Float4Holder left;
      @Param Float4Holder right;
      @Output BitHolder out;

      public void setup() {}

      public void eval() {

          out.value = left.value != right.value ? 1 : 0;

      }
  }

}

==> drill-java-exec-1.22.0.jar: org/apache/drill/exec/expr/fn/impl/GCompareFloat4VsFloat4.java <==

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.drill.exec.expr.fn.impl;

import org.apache.drill.common.FunctionNames;
import org.apache.drill.exec.expr.DrillSimpleFunc;
import org.apache.drill.exec.expr.annotations.FunctionTemplate;
import org.apache.drill.exec.expr.annotations.FunctionTemplate.NullHandling;
import org.apache.drill.exec.expr.annotations.Output;
import org.apache.drill.exec.expr.annotations.Param;
import org.apache.drill.exec.expr.fn.FunctionGenerationHelper;
import org.apache.drill.exec.expr.fn.impl.ByteFunctionHelpers;
import org.apache.drill.exec.expr.holders.*;
import org.apache.drill.exec.record.RecordBatch;
import javax.inject.Inject;
import io.netty.buffer.DrillBuf;

/*
 * This class is generated using freemarker and the ComparisonFunctions.java template.
 */

@SuppressWarnings("unused")
public class GCompareFloat4VsFloat4 {



  @FunctionTemplate(name = FunctionGenerationHelper.COMPARE_TO_NULLS_HIGH,
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.INTERNAL)
  public static class GCompareFloat4VsFloat4NullHigh implements DrillSimpleFunc {

    @Param Float4Holder left;
    @Param Float4Holder right;
    @Output IntHolder out;

    public void setup() {}

    public void eval() {
     outside:
      {


    // NaN is the biggest possible value, and NaN == NaN
      if(Double.isNaN(left.value) && Double.isNaN(right.value)) {
        out.value = 0;
      } else if (!Double.isNaN(left.value) && Double.isNaN(right.value)) {
        out.value = -1;
      } else if (Double.isNaN(left.value) && !Double.isNaN(right.value)) {
        out.value = 1;
      } else {
        out.value = left.value < right.value ? -1 : (left.value == right.value ? 0 : 1);
      }


      } // outside
    }
  }

  @FunctionTemplate(name = FunctionGenerationHelper.COMPARE_TO_NULLS_LOW,
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.INTERNAL)
  public static class GCompareFloat4VsFloat4NullLow implements DrillSimpleFunc {

    @Param Float4Holder left;
    @Param Float4Holder right;
    @Output IntHolder out;

    public void setup() {}

    public void eval() {
     outside:
      {


    // NaN is the biggest possible value, and NaN == NaN
      if(Double.isNaN(left.value) && Double.isNaN(right.value)) {
        out.value = 0;
      } else if (!Double.isNaN(left.value) && Double.isNaN(right.value)) {
        out.value = -1;
      } else if (Double.isNaN(left.value) && !Double.isNaN(right.value)) {
        out.value = 1;
      } else {
        out.value = left.value < right.value ? -1 : (left.value == right.value ? 0 : 1);
      }


      } // outside
    }
  }



  @FunctionTemplate(name = FunctionGenerationHelper.COMPARE_TO_NULLS_HIGH,
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.INTERNAL)
  public static class GCompareFloat4VsNullableFloat4NullHigh implements DrillSimpleFunc {

    @Param Float4Holder left;
    @Param NullableFloat4Holder right;
    @Output IntHolder out;

    public void setup() {}

    public void eval() {
     outside:
      {

      if ( right.isSet == 0 ) {
        out.value = -1;
        break outside;
      }

    // NaN is the biggest possible value, and NaN == NaN
      if(Double.isNaN(left.value) && Double.isNaN(right.value)) {
        out.value = 0;
      } else if (!Double.isNaN(left.value) && Double.isNaN(right.value)) {
        out.value = -1;
      } else if (Double.isNaN(left.value) && !Double.isNaN(right.value)) {
        out.value = 1;
      } else {
        out.value = left.value < right.value ? -1 : (left.value == right.value ? 0 : 1);
      }


      } // outside
    }
  }

  @FunctionTemplate(name = FunctionGenerationHelper.COMPARE_TO_NULLS_LOW,
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.INTERNAL)
  public static class GCompareFloat4VsNullableFloat4NullLow implements DrillSimpleFunc {

    @Param Float4Holder left;
    @Param NullableFloat4Holder right;
    @Output IntHolder out;

    public void setup() {}

    public void eval() {
     outside:
      {

      if ( right.isSet == 0 ) {
        out.value = 1;
        break outside;
      }

    // NaN is the biggest possible value, and NaN == NaN
      if(Double.isNaN(left.value) && Double.isNaN(right.value)) {
        out.value = 0;
      } else if (!Double.isNaN(left.value) && Double.isNaN(right.value)) {
        out.value = -1;
      } else if (Double.isNaN(left.value) && !Double.isNaN(right.value)) {
        out.value = 1;
      } else {
        out.value = left.value < right.value ? -1 : (left.value == right.value ? 0 : 1);
      }


      } // outside
    }
  }



  @FunctionTemplate(name = FunctionGenerationHelper.COMPARE_TO_NULLS_HIGH,
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.INTERNAL)
  public static class GCompareNullableFloat4VsFloat4NullHigh implements DrillSimpleFunc {

    @Param NullableFloat4Holder left;
    @Param Float4Holder right;
    @Output IntHolder out;

    public void setup() {}

    public void eval() {
     outside:
      {

        if ( left.isSet == 0 ) {
          out.value = 1;
          break outside;
        }

    // NaN is the biggest possible value, and NaN == NaN
      if(Double.isNaN(left.value) && Double.isNaN(right.value)) {
        out.value = 0;
      } else if (!Double.isNaN(left.value) && Double.isNaN(right.value)) {
        out.value = -1;
      } else if (Double.isNaN(left.value) && !Double.isNaN(right.value)) {
        out.value = 1;
      } else {
        out.value = left.value < right.value ? -1 : (left.value == right.value ? 0 : 1);
      }


      } // outside
    }
  }

  @FunctionTemplate(name = FunctionGenerationHelper.COMPARE_TO_NULLS_LOW,
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.INTERNAL)
  public static class GCompareNullableFloat4VsFloat4NullLow implements DrillSimpleFunc {

    @Param NullableFloat4Holder left;
    @Param Float4Holder right;
    @Output IntHolder out;

    public void setup() {}

    public void eval() {
     outside:
      {

        if ( left.isSet == 0 ) {
          out.value = -1;
          break outside;
        }

    // NaN is the biggest possible value, and NaN == NaN
      if(Double.isNaN(left.value) && Double.isNaN(right.value)) {
        out.value = 0;
      } else if (!Double.isNaN(left.value) && Double.isNaN(right.value)) {
        out.value = -1;
      } else if (Double.isNaN(left.value) && !Double.isNaN(right.value)) {
        out.value = 1;
      } else {
        out.value = left.value < right.value ? -1 : (left.value == right.value ? 0 : 1);
      }


      } // outside
    }
  }



  @FunctionTemplate(name = FunctionGenerationHelper.COMPARE_TO_NULLS_HIGH,
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.INTERNAL)
  public static class GCompareNullableFloat4VsNullableFloat4NullHigh implements DrillSimpleFunc {

    @Param NullableFloat4Holder left;
    @Param NullableFloat4Holder right;
    @Output IntHolder out;

    public void setup() {}

    public void eval() {
     outside:
      {

        if ( left.isSet == 0 ) {
          if ( right.isSet == 0 ) {
            out.value = 0;
            break outside;
          } else {
            out.value = 1;
            break outside;
          }
        } else if ( right.isSet == 0 ) {
          out.value = -1;
          break outside;
        }

    // NaN is the biggest possible value, and NaN == NaN
      if(Double.isNaN(left.value) && Double.isNaN(right.value)) {
        out.value = 0;
      } else if (!Double.isNaN(left.value) && Double.isNaN(right.value)) {
        out.value = -1;
      } else if (Double.isNaN(left.value) && !Double.isNaN(right.value)) {
        out.value = 1;
      } else {
        out.value = left.value < right.value ? -1 : (left.value == right.value ? 0 : 1);
      }


      } // outside
    }
  }

  @FunctionTemplate(name = FunctionGenerationHelper.COMPARE_TO_NULLS_LOW,
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.INTERNAL)
  public static class GCompareNullableFloat4VsNullableFloat4NullLow implements DrillSimpleFunc {

    @Param NullableFloat4Holder left;
    @Param NullableFloat4Holder right;
    @Output IntHolder out;

    public void setup() {}

    public void eval() {
     outside:
      {

        if ( left.isSet == 0 ) {
          if ( right.isSet == 0 ) {
            out.value = 0;
            break outside;
          } else {
            out.value = -1;
            break outside;
          }
        } else if ( right.isSet == 0 ) {
          out.value = 1;
          break outside;
        }

    // NaN is the biggest possible value, and NaN == NaN
      if(Double.isNaN(left.value) && Double.isNaN(right.value)) {
        out.value = 0;
      } else if (!Double.isNaN(left.value) && Double.isNaN(right.value)) {
        out.value = -1;
      } else if (Double.isNaN(left.value) && !Double.isNaN(right.value)) {
        out.value = 1;
      } else {
        out.value = left.value < right.value ? -1 : (left.value == right.value ? 0 : 1);
      }


      } // outside
    }
  }

   @FunctionTemplate(names = {FunctionNames.LT, "<"},
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.NULL_IF_NULL)
  public static class LessThanFloat4VsFloat4 implements DrillSimpleFunc {

    @Param Float4Holder left;
    @Param Float4Holder right;
    @Output BitHolder out;

    public void setup() {}

    public void eval() {

      // NaN is the biggest possible value, and NaN == NaN
      if (Double.isNaN(left.value) || ( Double.isNaN(left.value) && Double.isNaN(right.value))) {
        out.value=0;
      } else if (Double.isNaN(right.value) && !Double.isNaN(left.value)) {
        out.value = 1;
      } else {
        out.value = left.value < right.value ? 1 : 0;
      }
    }
  }

  @FunctionTemplate(names = {FunctionNames.LE, "<="},
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.NULL_IF_NULL)
  public static class LessThanEqFloat4VsFloat4 implements DrillSimpleFunc {

    @Param Float4Holder left;
    @Param Float4Holder right;
    @Output BitHolder out;

    public void setup() {}

    public void eval() {

      // NaN is the biggest possible value, and NaN == NaN
      if (Double.isNaN(right.value)){
        out.value = 1;
      } else if (!Double.isNaN(right.value) && Double.isNaN(left.value)) {
        out.value = 0;
      } else {
        out.value = left.value <= right.value ? 1 : 0;
      }
    }
  }

  @FunctionTemplate(names = {FunctionNames.GT, ">"},
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.NULL_IF_NULL)
  public static class GreaterThanFloat4VsFloat4 implements DrillSimpleFunc {

    @Param Float4Holder left;
    @Param Float4Holder right;
    @Output BitHolder out;

    public void setup() {}

    public void eval() {

      // NaN is the biggest possible value, and NaN == NaN
      if (Double.isNaN(right.value) || ( Double.isNaN(left.value) && Double.isNaN(right.value))) {
        out.value = 0;
      } else if (Double.isNaN(left.value) && !Double.isNaN(right.value)) {
        out.value = 1;
      } else {
        out.value = left.value > right.value ? 1 : 0;
      }
    }
  }

  @FunctionTemplate(names = {FunctionNames.GE, ">="},
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.NULL_IF_NULL)
  public static class GreaterThanEqFloat4VsFloat4 implements DrillSimpleFunc {

    @Param Float4Holder left;
    @Param Float4Holder right;
    @Output BitHolder out;

    public void setup() {}

    public void eval() {

      // NaN is the biggest possible value, and NaN == NaN
      if (Double.isNaN(left.value)){
        out.value=1;
      } else if (!Double.isNaN(left.value) && Double.isNaN(right.value)) {
        out.value = 0;
      } else {
        out.value = left.value >= right.value ? 1 : 0;
      }

    }
  }

  @FunctionTemplate(names = {FunctionNames.EQ, "==", "="},
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.NULL_IF_NULL)
  public static class EqualsFloat4VsFloat4 implements DrillSimpleFunc {

    @Param Float4Holder left;
    @Param Float4Holder right;
    @Output BitHolder out;

    public void setup() {}

    public void eval() {

      // NaN is the biggest possible value, and NaN == NaN
      if (Double.isNaN(left.value) && Double.isNaN(right.value)) {
        out.value = 1;
      } else {
        out.value = left.value == right.value ? 1 : 0;
      }
    }
  }

  @FunctionTemplate(names = {FunctionNames.NE, "<>", "!="},
                    scope = FunctionTemplate.FunctionScope.SIMPLE,
                    nulls = NullHandling.NULL_IF_NULL)
  public static class NotEqualsFloat4VsFloat4 implements DrillSimpleFunc {

    @Param Float4Holder left;
    @Param Float4Holder right;
    @Output BitHolder out;

    public void setup() {}

    public void eval() {

      // NaN is the biggest possible value, and NaN == NaN
      if (Double.isNaN(left.value) && Double.isNaN(right.value)) {
        out.value = 0;
      } else {
        out.value = left.value != right.value ? 1 : 0;
      }
    }
  }
}
